'''
Test various answer box types for proper XML rendering by `latex2edx/abox.py`
'''
from latex2edx.abox import AnswerBox
import unittest


class Test_Abox(unittest.TestCase):
    '''
    Class Test_Abox inherits the `unittest.TestCase` class and contains
    nine tests for the abox.py script
    '''

    def test_option1(self):
        '''
        Test dropdown option response
        '''
        abox = AnswerBox('type="option" expect="int" options="noneType","int","float"')
        xmlstr = abox.xmlstr
        print xmlstr
        self.assertIn('''<optioninput options="('noneType','int','float')" '''
                      '''correct="int"/>''', xmlstr)

    def test_string1(self):
        '''
        Test a string response with default settings
        '''
        abox = AnswerBox('type="string" expect="Michigan" size="20"')
        xmlstr = abox.xmlstr
        print xmlstr
        self.assertIn('<textline size="20"/>', xmlstr)
        self.assertIn('<stringresponse answer="Michigan" type="">', xmlstr)

    def test_string2(self):
        '''
        Test a string response with additional options
        '''
        abox = AnswerBox('type="string" expect="Michigan" size="20" options="ci regexp"')
        xmlstr = abox.xmlstr
        print xmlstr
        self.assertIn('<stringresponse answer="Michigan" type="ci regexp">', xmlstr)

    def test_numerical1(self):
        '''
        Test numerical response type
        '''
        abox = AnswerBox('''expect="3.14159" type="numerical" tolerance='0.01' inline=1''')
        xmlstr = abox.xmlstr
        print xmlstr
        self.assertIn('<numericalresponse inline="1" answer="3.14159">', xmlstr)
        self.assertIn('<textline inline="1">', xmlstr)
        self.assertIn('<responseparam type="tolerance" default="0.01"/>', xmlstr)

    def test_numerical2(self):
        '''
        Test numerical response type with units (the weight of 1 kilogram)
        '''
        abox = AnswerBox('''expect="9.81" type="numerical" tolerance="0.2"
                         inline="1" trailing_text="N"''')
        xmlstr = abox.xmlstr
        print xmlstr
        self.assertIn('<numericalresponse inline="1" answer="9.81">', xmlstr)
        self.assertIn('<textline inline="1" trailing_text="N">', xmlstr)
        self.assertIn('<responseparam type="tolerance" default="0.2"/>', xmlstr)

    def test_formula1(self):
        '''
        Test formula response
        '''
        abox = AnswerBox("""expect="(-b + sqrt(b^2-4*a*c))/(2*a)" type="formula" """ +
                         """samples="a,b,c@1,16,1:3,20,3#50" size="60" tolerance='0.01' inline='1' """ +
                         """math="1" feqin="1" """)
        xmlstr = abox.xmlstr
        print xmlstr
        self.assertIn('<formularesponse inline="1" type="cs" '
                      'samples="a,b,c@1,16,1:3,20,3#50" answer="(-b + '
                      'sqrt(b^2-4*a*c))/(2*a)">', xmlstr)
        self.assertIn('<formulaequationinput size="60" inline="1" math="1">', xmlstr)
        self.assertIn('<responseparam type="tolerance" default="0.01"/>', xmlstr)
        
    def test_oldmultichoice1(self):
        '''
        Test a multiple choice response requiring multiple selections
        '''
        abox = AnswerBox('type="oldmultichoice"  expect="Python","C++" options="Cobol","Pascal","Python","C++","Clu","Forth"')
        xmlstr = abox.xmlstr
        print xmlstr
        self.assertIn('<choiceresponse>', xmlstr)
        self.assertEqual(abox.xml.tag, 'choiceresponse')
        self.assertEqual(len(abox.xml.findall('.//choice')), 6)
        self.assertIn('<choice correct="true" name="3">', xmlstr)
        self.assertIn('<choice correct="true" name="4">', xmlstr)

    def test_custom1(self):
        '''
        Test a custom response refering to a `sumtest` python script
        '''
        abox = AnswerBox('''expect=""
        type="custom"
        answers="1","9"
        prompts="x = ","y = "
        cfn="sumtest"
        inline="1" ''')
        xmlstr = abox.xmlstr
        print xmlstr
        self.assertIn('<customresponse cfn="sumtest" inline="1" expect="">', xmlstr)
        self.assertIn('<p style="display:inline">x = <textline '
                      'correct_answer="1" inline="1"/></p>', xmlstr)
        self.assertIn('<p style="display:inline">y = <textline '
                      'correct_answer="9" inline="1"/></p>', xmlstr)
        self.assertIn('<br/>', xmlstr)

    def test_custom2(self):
        '''
        Test a custom response that uses javascript
        '''
        abox = AnswerBox('''expect="" type="jsinput" cfn="test_findep"
        width="650"
        height="555"
        gradefn="getinput"
        get_statefn="getstate"
        set_statefn="setstate"
        html_file="/static/html/ps3plot_btran1.html"''')
        xmlstr = abox.xmlstr
        print xmlstr
        self.assertIn('<customresponse cfn="test_findep" expect="">', xmlstr)
        self.assertIn('<jsinput width="650" height="555" gradefn="getinput" '
                      'get_statefn="getstate" set_statefn="setstate" '
                      'html_file="/static/html/ps3plot_btran1.html"/>', xmlstr)

    def test_custom3(self):
        '''
        Test a custom response with input formated as a code entry box
        '''
        abox = AnswerBox('''expect="test"
        rows=30 cols=80
        type="custom"
        cfn="sumtest"
        inline="1" ''')
        xmlstr = abox.xmlstr
        print xmlstr
        self.assertIn('<customresponse cfn="sumtest" inline="1" '
                      'expect="test">', xmlstr)
        self.assertIn('<textbox rows="30" cols="80" correct_answer="test" '
                      'inline="1"/>', xmlstr)

if __name__ == '__main__':
    unittest.main()



import os
import re
import contextlib
import unittest
import tempfile
import shutil
import json
from path import path  # needs path.py
from StringIO import StringIO

import latex2edx as l2emod
from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class TestBad_Script(unittest.TestCase):

    def test_bad_script1(self):
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example1_bad_script.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)

            try:
                l2e = latex2edx(nfn, output_dir=tmdir)
                l2e.convert()
            except Exception as err:
                pass

            print "Error = %s" % str(err)
            self.assertTrue(re.search('Error processing element script in file .*\.tex line 82', str(err)))

    def test_bad_script2(self):
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example7_bad_script.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)

            try:
                l2e = latex2edx(nfn, output_dir=tmdir)
                l2e.convert()
            except Exception as err:
                pass

            print "Error = %s" % str(err)
            self.assertTrue(re.search('Error processing element edxincludepy in file .*\.tex line 25', str(err)))

if __name__ == '__main__':
    unittest.main()



import os
import unittest
import sys
from path import path

# sys.path.append('../python_lib')
# from general_hint_system import *

# -----------------------------------------------------------------------------
# unittest entry


class TestGHS(unittest.TestCase):

    def test_policy1(self):
        assert(True)

    def test_ghs1(self):
        # import here, cause outside the eggs aren't loaded
        mydir = path(os.path.dirname(__file__)).dirname()
        libpath = path(os.path.abspath(mydir + '/python_lib'))
        if str(mydir) == '':
            libpath = '..' + libpath
        print "libpath=%s" % libpath
        os.chdir(libpath)
        import pytest
        errno = pytest.main('test_ghs.py')
        print "errno=%s" % errno
        assert(errno == 0)

if __name__ == '__main__':
    unittest.main()



import codecs
from plasTeX import Command, Environment, CountCommand
from plasTeX.Logging import getLogger
from plasTeX import Base

log = getLogger()
status = getLogger('status')


def ProcessOptions(options, document):
    context = document.context
    context.newcounter('edXchapter', initial=0)
    context.newcounter('edXsequential', resetby='edXchapter', initial=0)
    context.newcounter('edXvertical', resetby='edXsequential', initial=0)


class MyBaseCommand(Base.Command):  # add filename and linenum attributes
    def invoke(self, tex):
        Command.invoke(self, tex)
        self.attributes['filename'] = tex.filename
        self.attributes['linenum'] = tex.lineNumber


class MyBaseVerbatim(Base.verbatim):  # add filename and linenum attributes
    def invoke(self, tex):
        self.attributes['filename'] = tex.filename
        self.attributes['linenum'] = tex.lineNumber
        return Base.verbatim.invoke(self, tex)


class MyBaseEnvironment(Base.Environment):  # add filename and linenum attributes
    def invoke(self, tex):
        self.attributes['filename'] = tex.filename
        self.attributes['linenum'] = tex.lineNumber
        return Base.Environment.invoke(self, tex)


class edXcourse(Base.Environment):
    args = '{ number } { display_name } [ attrib_string:str ] self'


class EdXchapterStar(MyBaseEnvironment):
    macroName = 'edXchapter*'
    args = '{ display_name } [ attrib_string:str ] self'


class edXchapter(EdXchapterStar):
    macroName = 'edXchapter'
    counter = 'edXchapter'
    forcePars = True

    def invoke(self, tex):
        self.position = self.ownerDocument.context.counters[self.counter].value + 1
        return MyBaseEnvironment.invoke(self, tex)


class EdXsectionStar(MyBaseEnvironment):
    # turns into edXsequential
    macroName = 'edXsection*'
    args = '{ display_name } [ attrib_string:str ] self'


class edXsection(EdXsectionStar):
    # turns into edXsequential
    macroName = 'edXsection'
    counter = 'edXsequential'
    forcePars = True

    def invoke(self, tex):
        self.position = self.ownerDocument.context.counters[self.counter].value + 1
        return MyBaseEnvironment.invoke(self, tex)


class EdXsequentialStar(MyBaseEnvironment):
    macroName = 'edXsequential*'
    args = '{ display_name } [ attrib_string:str ] self'


class edXsequential(EdXsequentialStar):
    macroName = 'edXsequential'
    counter = 'edXsequential'
    forcePars = True

    def invoke(self, tex):
        self.position = self.ownerDocument.context.counters[self.counter].value + 1
        return MyBaseEnvironment.invoke(self, tex)


class EdXverticalStar(MyBaseEnvironment):
    macroName = 'edXvertical*'
    args = '{ display_name } [ attrib_string:str ] self'


class edXvertical(EdXverticalStar):
    macroName = 'edXvertical'
    counter = 'edXvertical'
    forcePars = True

    def invoke(self, tex):
        self.position = self.ownerDocument.context.counters[self.counter].value + 1
        return MyBaseEnvironment.invoke(self, tex)


class edXconditional(MyBaseEnvironment):
    macroName = 'edXconditional'
    args = '{ display_name } [ attrib_string:str ] self'


class edXabox(MyBaseCommand):
    args = 'self'


class edXinline(Base.Command):
    args = 'self'


class edXbr(Base.Command):
    args = 'self'


class edXvideo(Base.Command):
    args = '{ display_name } { youtube } [ attrib_string ] self'
    # args = 'self'


class edXlti(Base.Command):
    args = '{ display_name } { launch_url } { lti_id } [ attrib_string ] self'
    # args = 'self'


class edXdiscussion(Base.Command):
    args = '{ display_name } { attrib_string } self'


class includegraphics(Base.Command):
    args = '[ width ] self'


class edXcite(Base.Command):  # tooltip citation (appears onmoseover, using <a title="self" href="#"><sup>[ref]</sup></a>)
    args = '[ ref ] self'


class edXinclude(MyBaseCommand):  # include external XML file
    args = 'self'


class edXincludepy(MyBaseCommand):  # include external python file (puts inside <script>)
    args = 'self'


class edXdndtex(MyBaseCommand):  # insert external drag-and-drop problem (should point to latex2dnd tex file)
    args = '[ attrib_string ] self'

    def invoke(self, tex):
        Command.invoke(self, tex)
        print "  --> edXdndtex in %s: dndtex=%s, line=%s" % (tex.filename, self.attributes['self'].source, tex.lineNumber)


class edXshowhide(Base.Environment):  # block of text to be hidden by default, but with clickable "show"
    args = ' { description } self'


class edXscript(MyBaseVerbatim):
    macroName = "edXscript"


class endedXscript(Base.endverbatim):
    macroName = "endedXscript"


class edXanswer(Base.verbatim):
    macroName = "edXanswer"


class endedXanswer(Base.endverbatim):
    macroName = "endedXanswer"


class edXjavascript(Base.verbatim):
    macroName = "edXjavascript"


class endedXjavascript(Base.endverbatim):
    macroName = "endedXjavascript"


class edXmath(Base.verbatim):
    macroName = "edXmath"


class endedXmath(Base.endverbatim):
    macroName = "endedXmath"


class edXxml(Base.Command):
    args = 'self'


class edXproblem(MyBaseEnvironment):
    args = '{ display_name } { attrib_string } self'


class edXtext(MyBaseEnvironment):  # indicates HTML file to be included (ie <html...> in course.xml)
    args = '{ display_name } [ attrib_string:str ] self'


class edXsolution(MyBaseEnvironment):
    args = 'self'
    # note: cannot have \[ immediately after \begin{edXsolution}


class section(Base.Command):
    args = '* self'


class subsection(Base.Command):
    args = '* self'


class subsubsection(Base.Command):
    args = '* self'


class label(Base.Command):
    args = 'self'


class ref(Base.Command):
    args = 'self'


class toclabel(Base.Command):
    args = 'self'


class tocref(Base.Command):
    args = 'self'


class index(Base.Command):
    args = 'self'


class href(Base.Command):
    args = '{ url } { self }'


class textwidth(Base.Command):
    args = ''


class edXaskta(Base.Command):
    # ask TA link
    args = 'self'


class input(Command):
    """ verbose version of \\input
    """
    args = 'name:str'

    def invoke(self, tex):
        a = self.parse(tex)
        # print "myinput a=%s, tex=%s" % (a,tex)
        try:
            path = tex.kpsewhich(a['name'])
        except Exception, msg:
            # print "myinput kpsewhich error=%s" % msg
            path = a['name']
            if not path.endswith('.tex'):
                path += '.tex'

        try:
            print "\n----------------------------------------------------------------------------- Input [%s]" % path
            status.info(' ( %s ' % path)
            encoding = self.config['files']['input-encoding']
            tex.input(codecs.open(path, 'r', encoding, 'replace'))
            status.info(' ) ')

        except (OSError, IOError), msg:
            print "myinput error=%s" % msg
            log.warning(msg)
            status.info(' ) ')


class edXgitlink(Command):
    '''
    Output link to github file with line number of specific place inside file
    '''
    args = '{url_root}{name}'

    def invoke(self, tex):
        Command.invoke(self, tex)
        self.attributes['filename'] = tex.filename
        self.attributes['linenum'] = tex.lineNumber
        self.attributes['url'] = '%s/%s#L%s' % (self.attributes['url_root'].source, tex.filename, tex.lineNumber)
        # print "root=%s, name=%s" % (self.attributes['url_root'].source, self.attributes['name'].source)
        print "  --> edXgitlink: file=%s, line=%s, url=%s" % (tex.filename, tex.lineNumber, self.attributes['url'])



import os
import unittest
from StringIO import StringIO

from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory
from latex2edx.course_tests import AnswerBoxUnitTest, CourseUnitTestSet

class TestCourseUnitTests(unittest.TestCase):

    def test_latex2edx_cutset1(self):
        with make_temp_directory() as tmdir:
            os.chdir(tmdir)
            ofn = "testcuts.yaml"
            tex = r'''\begin{edXproblem}{A problem}{url_name="a_problem"}
    
                   \edXabox{type="option" options="red","green","blue" expect="red"}
    
                   \end{edXproblem}'''
            l2e = latex2edx(tmdir + '/test.tex', latex_string=tex, add_wrap=True,
                            do_images=False, output_dir=tmdir, output_cutset=ofn)
            xhtml = l2e.xhtml
            assert('<optionresponse' in  xhtml)
    
            xmlstr = l2e.xml
            cutset = CourseUnitTestSet(fn=ofn)
            self.assertEqual(len(cutset.tests), 1)
            self.assertEqual(cutset.tests[0].url_name, "a_problem")
            self.assertEqual(cutset.tests[0].responses, ["red"])
            self.assertEqual(cutset.tests[0].expected, ["correct"])

    def test_latex2edx_cutset1a(self):
        with make_temp_directory() as tmdir:
            os.chdir(tmdir)
            ofn = "testcuts.yaml"
            tex = r'''\begin{edXproblem}{A problem}{url_name="a_problem"}
    
                   \edXabox{type="option" options="red","green","blue" expect="red" test_fail="green"}
    
                   \end{edXproblem}'''
            l2e = latex2edx(tmdir + '/test.tex', latex_string=tex, add_wrap=True,
                            do_images=False, output_dir=tmdir, output_cutset=ofn)
            xhtml = l2e.xhtml
            assert('<optionresponse' in  xhtml)
    
            xmlstr = l2e.xml
            cutset = CourseUnitTestSet(fn=ofn)
            self.assertEqual(len(cutset.tests), 2)
            self.assertEqual(cutset.tests[1].url_name, "a_problem")
            self.assertEqual(cutset.tests[1].responses, ["red"])
            self.assertEqual(cutset.tests[1].expected, ["correct"])
            self.assertEqual(cutset.tests[0].url_name, "a_problem")
            self.assertEqual(cutset.tests[0].responses, ["green"])
            self.assertEqual(cutset.tests[0].expected, "incorrect")

    def test_latex2edx_cutset1b(self):
        with make_temp_directory() as tmdir:
            os.chdir(tmdir)
            ofn = "testcuts.yaml"
            tex = r'''\begin{edXproblem}{A problem}{url_name="a_problem"}
    
                   \edXabox{type="option" options="<red>","green","blue" expect="<red>" 
                            test_pass="<ruby>" test_fail="<green>"}
    
                   \end{edXproblem}'''
            l2e = latex2edx(tmdir + '/test.tex', latex_string=tex, add_wrap=True,
                            do_images=False, output_dir=tmdir, output_cutset=ofn)
            xmlstr = l2e.xml
            cutset = CourseUnitTestSet(fn=ofn)
            self.assertEqual(len(cutset.tests), 2)
            self.assertEqual(cutset.tests[0].url_name, "a_problem")
            self.assertEqual(cutset.tests[0].responses, ["<ruby>"])
            self.assertEqual(cutset.tests[0].expected, "correct")

    def test_latex2edx_cutset2(self):
        with make_temp_directory() as tmdir:
            os.chdir(tmdir)
            ofn = "testcuts.yaml"
            tex = r'''
                   \begin{edXproblem}{A problem}{url_name="a_problem"}

                   \edXabox{type="option" options="red","green","blue" expect="red"}
                   \end{edXproblem}

                   \begin{edXproblem}{A problem 2}{url_name="a_problem2"}

                   \edXabox{type="custom" expect="red" cfn="mytest"}
                   \end{edXproblem}
            '''
            l2e = latex2edx(tmdir + '/test.tex', latex_string=tex, add_wrap=True,
                            do_images=False, output_dir=tmdir, output_cutset=ofn)
            xhtml = l2e.xhtml
            assert('<optionresponse' in  xhtml)
    
            xmlstr = l2e.xml
            cutset = CourseUnitTestSet(fn=ofn)
            self.assertEqual(len(cutset.tests), 2)
            self.assertEqual(cutset.tests[1].url_name, "a_problem2")
            self.assertEqual(cutset.tests[1].responses, ["red"])
            self.assertEqual(cutset.tests[1].expected, ["correct"])

    def test_latex2edx_cutset3(self):
        with make_temp_directory() as tmdir:
            os.chdir(tmdir)
            ofn = "testcuts.yaml"
            tex = r'''
                   \begin{edXproblem}{A problem with two aboxes}{url_name="a_problem"}

                   \edXabox{type="option" options="red","green","blue" expect="red"}

                   \edXabox{type="custom" expect="42" cfn="mytest"}
                   \end{edXproblem}
            '''
            l2e = latex2edx(tmdir + '/test.tex', latex_string=tex, add_wrap=True,
                            do_images=False, output_dir=tmdir, output_cutset=ofn)
            xhtml = l2e.xhtml
            assert('<optionresponse' in  xhtml)
    
            xmlstr = l2e.xml
            cutset = CourseUnitTestSet(fn=ofn)
            self.assertEqual(len(cutset.tests), 1)
            self.assertEqual(cutset.tests[0].url_name, "a_problem")
            self.assertEqual(cutset.tests[0].responses, ["red", "42"])
            self.assertEqual(cutset.tests[0].expected, ["correct", "correct"])

    def test_latex2edx_cutset4(self):
        with make_temp_directory() as tmdir:
            os.chdir(tmdir)
            ofn = "testcuts.yaml"
            tex = r'''
                   \begin{edXproblem}{A problem with two aboxes}{url_name="a_problem"}

                   \edXabox{type="option" options="red","green","blue" expect="red" test_pass="red" test_fail="green"}

                   \edXabox{type="custom" expect="42" cfn="mytest" test_fail="11" test_pass="42"}
                   \end{edXproblem}
            '''
            l2e = latex2edx(tmdir + '/test.tex', latex_string=tex, add_wrap=True,
                            do_images=False, output_dir=tmdir, output_cutset=ofn)
            xhtml = l2e.xhtml
            assert('<optionresponse' in  xhtml)
    
            xmlstr = l2e.xml
            cutset = CourseUnitTestSet(fn=ofn)
            self.assertEqual(len(cutset.tests), 4)
            self.assertEqual(cutset.tests[0].url_name, "a_problem")
            self.assertEqual(cutset.tests[0].responses, ["red", "11"])
            self.assertEqual(cutset.tests[0].expected, ["correct", "incorrect"])

    def test_latex2edx_cutset5(self):
        with make_temp_directory() as tmdir:
            os.chdir(tmdir)
            ofn = "testcuts.yaml"
            tex = r'''\begin{edXproblem}{A problem}{url_name="a_problem"}
    
                   \edXabox{type="option" options="red","green","blue" expect="red"}
    
            \edXinline{What is $n$ for $C^\perp$?~} \edXabox{type=symbolic size=10 expect="3"  inline="1"}

            \edXinline{$d(C) = $} \edXabox{type=numerical size=10 expect="5" inline="1"}

            \edXinline{$\Delta t = $} \edXabox{type="custom" expect="pi" cfn=sympy_formula_check inline="1"}

            \edXinline{$\Delta t = $} \edXabox{type="formula" expect="pi" inline="1" samples='pi@1:10#10'}

                   \end{edXproblem}'''
            l2e = latex2edx(tmdir + '/test.tex', latex_string=tex, add_wrap=True,
                            do_images=False, output_dir=tmdir, output_cutset=ofn)
            xmlstr = l2e.xml
            cutset = CourseUnitTestSet(fn=ofn)
            self.assertEqual(len(cutset.tests), 1)
            self.assertEqual(cutset.tests[0].url_name, "a_problem")
            self.assertEqual(len(cutset.tests[0].responses), 5)

if __name__ == '__main__':
    unittest.main()



import os
import unittest
from lxml import etree
from StringIO import StringIO

from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class MakeTeX(object):
    def __init__(self, tex):
        buf = """\\documentclass[12pt]{article}\n\\usepackage{edXpsl}\n\n\\begin{document}"""
        buf += tex
        buf += "\\end{document}"
        self.buf = buf

    @property
    def fp(self):
        return StringIO(self.buf)


class TestText(unittest.TestCase):

    def test_text1(self):

        tex = ('\\begin{edXcourse}{1.00x}{1.00x Fall 2013}[url_name=2013_Fall]\n'
               '\\begin{edXchapter}{Unit 1}[start="2013-11-22"]\n'
               '\\begin{edXsection}{Introduction}\n'
               '\\begin{edXtext}{My Name}[url_name=text_url_name]\n'
               'Hello world!\n\n'
               '\\end{edXtext}\n'
               '\\end{edXsection}\n'
               '\\end{edXchapter}\n'
               '\\end{edXcourse}\n'
               )

        with make_temp_directory() as tmdir:
            os.chdir(tmdir)
            fp = MakeTeX(tex).fp
            l2e = latex2edx(tmdir + '/test.tex', fp=fp, do_images=False, output_dir=tmdir)
            l2e.xhtml2xbundle()
            # print "xbundle = "
            # print str(l2e.xb)
            # print

            # self.assertIn(r'<html display_name="My Name" url_name="text_url_name">', str(l2e.xb))

            xml = etree.fromstring(str(l2e.xb))
            html = xml.find('.//html')
            self.assertTrue(html.get('display_name') == 'My Name')
            self.assertTrue(html.get('url_name') == 'text_url_name')

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Answer Box class
#
# object representation of abox, used in Tutor2, now generalized to latex and word input formats.
# 13-Aug-12 ichaung: merge in sari's changes
# 13-Aug-12 ichuang: cleaned up, does more error checking, includes stub for shortanswer
#                    note that shortanswer can be implemented right now using customresponse and textbox
# 04-Sep-12 ichuang: switch from shlex to FSM, merge in changes for math and inline from 8.21
# 13-Oct-12 ichuang: remove csv entirely, use FSM for splitting options instead
# 20-Jan-13 ichuang: add formularesponse
# 23-Jan-13 ichuang: add multiple-line customresponse, with proper inline and math handling

import os, sys, string, re
import hashlib	# for unique abox ID
# import shlex	# for split keeping quoted strings intact
# import csv	# for splitting quoted options

from lxml import etree


class AnswerBox(object):
    def __init__(self, aboxstr, config=None, context=None, verbose=False):
        '''
        Parse a TUT abox and produce edX XML for a problem responsetype.

        Examples:
        -----------------------------------------------------------------------------
        <abox type="option" expect="float" options=" ","noneType","int","float" />
        
        <optionresponse>
        <optioninput options="('noneType','int','float')"  correct="int">
        </optionresponse>
        
        -----------------------------------------------------------------------------
        <abox type="string" expect="Michigan" options="ci" />
        
        <stringresponse answer="Michigan" type="ci">
        <textline size="20" />
        </stringresponse>
        
        -----------------------------------------------------------------------------
        <abox type="custom" expect="(3 * 5) / (2 + 3)" cfn="eq" />
        
        <customresponse cfn="eq">
        <textline size="40" correct_answer="(3 * 5) / (2 + 3)"/><br/>
        </customresponse>
        
        -----------------------------------------------------------------------------
        <abox type="custom" expect="20" answers="11","9" prompts="Integer 1:","Integer 2:" inline="1" cfn="test_add" />

        <customresponse cfn="test_add" expect="20" inline="1">
            <p style="display:inline">Integer 1:<textline correct_answer="11" inline="1"/></p>
            <br/>
            <p style="display:inline">Integer 2:<textline correct_answer="9" inline="1"/></p>
        </customresponse>

        -----------------------------------------------------------------------------
        <abox type="jsinput" expect="(3 * 5) / (2 + 3)" cfn="eq" gradefn="gradefn" height="500"
               get_statefn="getstate" set_statefn="setstate" html_file="/static/jsinput.html"/>
        
        <customresponse cfn="eq" expect="(3 * 5) / (2 + 3)">
            <jsinput gradefn="gradefn"
                height="500"
                get_statefn="getstate"
                set_statefn="setstate"
                html_file="/static/jsinput.html"/>
        </customresponse>
        
        -----------------------------------------------------------------------------
        <abox type="numerical" expect="3.141" tolerance="5%" />
        
        <numericalresponse answer="5.0">
        <responseparam type="tolerance" default="5%" name="tol" description="Numerical Tolerance" />
        <textline />
        </numericalresponse>
        
        -----------------------------------------------------------------------------
    <abox type="multichoice" expect="Yellow" options="Red","Green","Yellow","Blue" />

        <multiplechoiceresponse direction="vertical" randomize="yes">
         <choicegroup type="MultipleChoice">
            <choice location="random" correct="false" name="red">Red</choice>
            <choice location="random" correct="true" name="green">Green</choice>
            <choice location="random" correct="false" name="yellow">Yellow</choice>
            <choice location="bottom" correct="false" name="blue">Blue</choice>
         </choicegroup>
        </multiplechoiceresponse>
        -----------------------------------------------------------------------------
    <abox type="oldmultichoice" expect="1","3" options="0","1","2","3","4" />

        <choiceresponse>
          <checkboxgroup>
            <choice correct="false"><text>0</text></choice>
            <choice correct="true"><text>1</text></choice>
            <choice correct="false"><text>2</text></choice>
            <choice correct="true"><text>3</text></choice>
            <choice correct="false"><text>4</text></choice>
          </checkboxgroup>
        </choiceresponse>
        -----------------------------------------------------------------------------
        <abox type="formula" expect="m*c^2" samples="m,c@1,2:3,4#10" intype="cs" size="40" math="1" tolerance="0.01" feqin="1" />

        format of samples:  <variables>@<lower_bounds>:<upper_bound>#<num_samples

        * variables    - a set of variables that are allowed as student input
        * lower_bounds - for every variable defined in variables, a lower
                         bound on the numerical tests to use for that variable
        * upper_bounds - for every variable defined in variables, an upper
                         bound on the numerical tests to use for that variable

        if feqin is given as an attribute, then a formulaequationinput is used instead
        of textline, for the input element.

        <formularesponse type="cs" samples="m,c@1,2:3,4#10" answer="m*c^2">
            <responseparam type="tolerance" default="0.01"/>
            <textline size="40" math="1" />
        </formularesponse>

        -----------------------------------------------------------------------------
        Adaptive hints:
        
        define the hints as a dict in an included python script, and give the name
        of that dict as the parameter "hints".  Works inside customresponse,
        optionresponse, and multiple choice problems, within latex2edx.
        
        latex2edx automatically translates <ed_general_hint_system/> into an import
        of the general_hint_system.py python code.

        Thus, this input:

        <abox type="custom" expect="(3 * 5) / (2 + 3)" cfn="eq" hints="hint1"/>
        
        produces:

        <edx_general_hint_system />

        <script type="text/python">
        do_hints_for_hint1 = HintSystem(hints=hint1).check_hint
        </script>

        <customresponse cfn="eq">
        <textline size="40" correct_answer="(3 * 5) / (2 + 3)"/><br/>
        <hintgroup hintfn="do_hints_for_hint1">
        </customresponse>

        -----------------------------------------------------------------------------
        Unit tests:

        Unit tests for answer boxes can be generated with latex2edx and aboxes, by using
        arguments test_pass, test_fail, and test_spec, e.g.

        \edXabox{type="custom" expect=10 cfn=mytest test_fail=12 test_pass=10}

        should generate two unit test caes for the answer box - one which is expected 
        to be graded "incorrect" (the test_fail case), and one which is expected
        to be graded "correct" (the test_pass case).

        -----------------------------------------------------------------------------

        context is used for error reporting, and provides context like the line number and
        filename where the abox is located.

        '''
        self.aboxstr = aboxstr
        self.context = context
        self.verbose = verbose
        self.tests = []
        self.has_test_pass = False
        if config is None:
            self.config = {}
        else:
            self.config = config
        self.xml = self.abox2xml(aboxstr)
        self.xmlstr_no_hints = etree.tostring(self.xml)
        self.xmlstr = self.hint_extras + self.xmlstr_no_hints
        self.xmlstr_no_hints = self.xmlstr_no_hints.strip()	# cannonicalize, since it's may be used as a key 
        
    def abox2xml(self, aboxstr):
        if aboxstr.startswith('abox '): aboxstr = aboxstr[5:]
        s = aboxstr
        s = s.replace(' in_check= ', ' ')

        # unique ID for this abox, using hash
        try:
            aboxid = hashlib.sha1(aboxstr).hexdigest()[:10]
        except Exception as err:
            aboxid = hashlib.sha1(aboxstr.encode('utf8')).hexdigest()[:10]

        # parse answer box arguments into dict
        abargs = self.abox_args(s)
        self.abargs = abargs

        type2response = {'custom': 'customresponse',
                         'external': 'externalresponse',
                         'code': 'coderesponse',
                         'oldmultichoice': 'choiceresponse',
                         'multichoice': 'multiplechoiceresponse',
                         'numerical': 'numericalresponse',
                         'option': 'optionresponse',
                         'formula': 'formularesponse',
                         'shortans': 'shortanswerresponse',
                         'shortanswer': 'shortanswerresponse',
                         'string': 'stringresponse',
                         'symbolic': 'symbolicresponse',
                         'image': 'imageresponse',
                         'jsinput': 'customresponse_jsinput',
                         'config': 'config',	# special for setting default config parameters
                         }

        if 'type' in abargs and abargs['type'] in type2response:
            abtype = type2response[abargs['type']]
        elif 'tests' in abargs:
            abtype = 'externalresponse'
        elif 'type' not in abargs and 'options' in abargs:
            abtype = 'optionresponse'
        elif 'cfn' in abargs:
            abtype = 'customresponse'
        else:
            abtype = 'symbolicresponse'  # default
        
        abxml = etree.Element(abtype)
        script_code = None

        # if config specifies default parameters for this type of answer box, then use them
        if abtype in self.config:
            for k,v in self.config[abtype].iteritems():
                if k not in self.abargs:
                    self.abargs[k] = v
            print "abargs = ", abargs

        if abtype == 'optionresponse':
            self.require_args(['expect'])
            oi = etree.Element('optioninput')
            optionstr, options = self.get_options(abargs)
            expected = self.stripquotes(abargs['expect'])
            oi.set('options', optionstr)
            oi.set('correct', expected)
            abxml.append(oi)
            self.copy_attrib(abargs, 'inline', abxml)
            self.copy_attrib(abargs, 'inline', oi)
            try:
                expect_idx = options.index(expected)
            except Exception as err:
                raise Exception("[abox] Error: expected=%s is not one of the options=%s for aboxstr=%s" % (expected, options, aboxstr))
            if not self.has_test_pass:		# generate unit test if no explicit tests specified in abox arguments
                self.tests.append({'responses': [expected],
                                   'expected': ['correct'],
                                   })
            
        if abtype == 'multiplechoiceresponse':
            self.require_args(['expect', 'options'])
            cg = etree.SubElement(abxml, 'choicegroup')
            cg.set('direction', 'vertical')
            optionstr, options = self.get_options(abargs)
            expectstr, expectset = self.get_options(abargs, arg='expect')
            cnt = 1
            correctset = []
            for op in options:
                choice = etree.SubElement(cg, 'choice')
                choice.set('correct', 'true' if op in expectset else 'false')
                choice.set('name', str(cnt))
                choice.append(etree.XML("<text> %s</text>" % op))
                if op in expectset:
                    correctset.append(cnt)
                cnt += 1
            if not self.has_test_pass:		# generate unit test if no explicit tests specified in abox arguments
                self.tests.append({'responses': ["choice_%d" % x for x in correctset],
                                   'expected': ['correct'] * len(correctset),
                                   })
            
        if abtype == 'choiceresponse':
            self.require_args(['expect', 'options'])
            cg = etree.SubElement(abxml, 'checkboxgroup')
            optionstr, options = self.get_options(abargs)
            expectstr, expects = self.get_options(abargs, 'expect')
            cnt = 1
            correctset = []
            if self.verbose:
                print "[abox.py] oldmultichoice: options=/%s/, expects=/%s/" % (options, expects)
            for op in options:
                choice = etree.SubElement(cg, 'choice')
                choice.set('correct', 'true' if (op in expects) else 'false')
                choice.set('name', str(cnt))
                choice.append(etree.XML("<text>%s</text>" % op))
                if op in expects:
                    correctset.append(cnt)
                cnt += 1
            if not self.has_test_pass:		# generate unit test if no explicit tests specified in abox arguments
                self.tests.append({'responses': ["choice_%d" % x for x in correctset],
                                   'expected': ['correct'] * len(correctset),
                                   })

        elif abtype == 'shortanswerresponse':
            print "[latex2html.abox] Warning - short answer response quite yet implemented in edX!"
            if 1:
                tb = etree.Element('textbox')
                self.copy_attrib(abargs, 'rows', tb)
                self.copy_attrib(abargs, 'cols', tb)
                abxml.append(tb)
                abxml.tag = 'customresponse'
                self.require_args(['expect', 'cfn'])
                abxml.set('cfn', self.stripquotes(abargs['cfn']))
                self.copy_attrib(abargs, 'expect', abxml)
                
            else:
                abxml.tag = 'stringresponse'    # change to stringresponse for now (FIXME)
                tl = etree.Element('textline')
                if 'size' in abargs:
                    tl.set('size', self.stripquotes(abargs['size']))
                else:
                    tl.set('size', '80')
                self.copy_attrib(abargs, 'trailing_text', tl)
                abxml.append(tl)
                abxml.set('answer', 'unknown')
                self.copy_attrib(abargs, 'inline', tl)
                self.copy_attrib(abargs, 'inline', abxml)
            
        elif abtype == 'stringresponse':
            self.require_args(['expect'])
            tl = etree.Element('textline')
            if 'size' in abargs:
                tl.set('size', self.stripquotes(abargs['size']))
            self.copy_attrib(abargs, 'trailing_text', tl)
            abxml.append(tl)
            answer = self.stripquotes(abargs['expect'])
            abxml.set('answer', answer)
            if 'options' in abargs:
                abxml.set('type', self.stripquotes(abargs['options']))
            else:
                abxml.set('type', '')
            self.copy_attrib(abargs, 'inline', tl)
            self.copy_attrib(abargs, 'inline', abxml)
            if not self.has_test_pass:		# generate unit test if no explicit tests specified in abox arguments
                self.tests.append({'responses': [answer],
                                   'expected': ['correct'],
                                   })

        elif abtype == 'customresponse':
            self.require_args(['expect', 'cfn'])
            abxml.set('cfn', self.stripquotes(abargs['cfn']))
            self.copy_attrib(abargs, 'inline', abxml)
            self.copy_attrib(abargs, 'expect', abxml)
            self.copy_attrib(abargs, 'options', abxml)
            if abxml.get('options', ''):
                abxml.set('cfn_extra_args', 'options')  # tells sandbox to include 'options' in cfn call arguments
            if 'answers' not in abargs:
                answers = [self.stripquotes(abargs['expect'])]
            else:   # multiple inputs for this customresponse
                ansstr, answers = self.get_options(abargs, 'answers')
            if 'prompts' in abargs:
                promptstr, prompts = self.get_options(abargs, 'prompts')
            else:
                prompts = ['']
            if not len(prompts) == len(answers):
                msg = "Error: number of answers and prompts must match in:"
                msg += aboxstr
                msg += "\nabox located: %s\n" % self.context
                raise Exception(msg)
                # sys.exit(-1)

            # if wrapclass defined, then use that class to transform "expect" and "ans"
            # before it is processed by cfn
            wrapclass = abargs.get('wrapclass', '')
            if wrapclass:
                code_lines = []
                wid = "wrap_%s" % (aboxid)
                the_import = abargs.get("import", "")
                if the_import:
                    code_lines.append("import %s" % the_import)
                code_lines.append("%s = %s" % (wid, wrapclass))
                orig_answers = answers[:]	# copy of answers
                new_answers = []
                acnt = 0
                # wrap displayed answers
                for ans in answers:
                    acnt += 1
                    aid = "ans_%s_%d" % (aboxid, acnt)
                    code_lines.append("%s = %s.answer('''%s''')" % (aid, wid,ans))
                    new_answers.append("$%s" % aid)
                answers = new_answers

                # wrap expected answer
                expect = abxml.get("expect")
                code_lines.append("expect_%s = %s.answer('''%s''')" % (wid, wid, expect))
                abxml.set("expect", "$expect_%s" % wid)

                # wrap the check function
                code_lines.append("")
                code_lines.append("@%s.grader" % wid)
                code_lines.append("def cfn_%s(expect, ans, **kwargs):" % wid)
                code_lines.append("    return %s(expect, ans, **kwargs)" % abxml.get('cfn'))
                abxml.set("cfn", "cfn_%s" % wid)	# use wrapped check function

                script_code = '\n'.join(code_lines)

            cnt = 0
            for ans, prompt in zip(answers, prompts):
                if 'rows' in abargs:
                    tl = etree.Element('textbox')
                    self.copy_attrib(abargs, 'rows', tl)
                    self.copy_attrib(abargs, 'cols', tl)
                else:
                    tl = etree.Element('textline')
                    self.copy_attrib(abargs, 'size', tl)
                tl.set('correct_answer', ans)
                self.copy_attrib(abargs, 'trailing_text', tl)
                self.copy_attrib(abargs, 'inline', tl)
                self.copy_attrib(abargs, 'math', tl)
                self.copy_attrib(abargs, 'preprocessorClassName', tl)
                self.copy_attrib(abargs, 'preprocessorSrc', tl)
                if prompt:
                    elem = etree.Element('p')
                    if 'inline' in abargs:
                        elem.set('style', 'display:inline')
                    elem.text = prompt + " "
                    elem.append(tl)
                else:
                    elem = tl
                if cnt > 0:
                    abxml.append(etree.Element('br'))   # linebreak between boxes if multiple
                abxml.append(elem)
                cnt += 1

            if not self.has_test_pass:		# generate unit test if no explicit tests specified in abox arguments
                self.tests.append({'responses': answers,
                                   'expected': ['correct'] * len(answers),
                                   'box_indexes': zip([0]*len(answers), range(len(answers))),
                                   })
                    
        elif abtype == 'customresponse_jsinput':
            abxml.tag = 'customresponse'
            self.require_args(['expect', 'cfn'])
            abxml.set('cfn', self.stripquotes(abargs['cfn']))
            self.copy_attrib(abargs, 'expect', abxml)
            self.copy_attrib(abargs, 'options', abxml)
            if abxml.get('options', ''):
                abxml.set('cfn_extra_args', 'options')  # tells sandbox to include 'options' in cfn call arguments

            js = etree.Element('jsinput')
            jsattribs = ['width', 'height', 'gradefn', 'get_statefn', 'set_statefn', 'html_file', 'sop']
            for jsa in jsattribs:
                self.copy_attrib(abargs, jsa, js)
            abxml.append(js)
                    
        elif abtype == 'externalresponse' or abtype == 'coderesponse':
            if 'url' in abargs:
                self.copy_attrib(abargs, 'url', abxml)
            tb = etree.Element('textbox')
            self.copy_attrib(abargs, 'rows', tb)
            self.copy_attrib(abargs, 'cols', tb)
            self.copy_attrib(abargs, 'mode', tb)
            self.copy_attrib(abargs, 'tabsize', tb)
            self.copy_attrib(abargs, 'tests', abxml)
            self.copy_attrib(abargs, 'queuename', abxml)
            abxml.append(tb)
            if abtype=="coderesponse":
                #
                # sample coderesponse:
                #   <coderesponse queuename="MITx-42.01x">
                #       <textbox rows="10" cols="80" mode="python" tabsize="4"/>
                #       <codeparam>
                #           <initial_display>
                #             # students please write your program here
                #             print ""
                #           </initial_display>
                #           <answer_display>
                #             print "hello world"
                #           </answer_display>
                #           <grader_payload>
                #           {"output": "hello world", "max_length": 2}
                #           </grader_payload>
                #       </codeparam>
                #   </coderesponse>
                #
                cp = etree.SubElement(abxml, "codeparam")
                cp_id = etree.SubElement(cp, "initial_display")
                cp_id.text = abargs.get("initial_display", "")
                cp_ad = etree.SubElement(cp, "answer_display")
                cp_ad.text = abargs.get("answer_display", "")
                cp_gp = etree.SubElement(cp, "grader_payload")
                cp_gp.text = abargs.get("grader_payload", "")
            # turn script to <answer> later

        elif abtype == 'numericalresponse':
            self.require_args(['expect'])
            self.copy_attrib(abargs, 'inline', abxml)
            tl = etree.Element('textline')
            self.copy_attrib(abargs, 'size', tl)
            self.copy_attrib(abargs, 'inline', tl)
            self.copy_attrib(abargs, 'math', tl)
            self.copy_attrib(abargs, 'trailing_text', tl)
            abxml.append(tl)
            self.copy_attrib(abargs, 'options', abxml)
            answer = self.stripquotes(abargs['expect'])
            # NOTE: The edX platform now allows mathematical expressions
            #       and constants in the expect field.
            # try:
            #     x = float(answer)
            # except Exception as err:
            #     if not answer[0] == '$':    # may also be a string variable (starts with $)
            #         print "Error - numericalresponse expects numerical expect value, for %s" % s
            #         raise
            abxml.set('answer', answer)
            rp = etree.SubElement(tl, "responseparam")
            rp.attrib['type'] = "tolerance"
            rp.attrib['default'] = abargs.get('tolerance') or "0.00001"
            if not self.has_test_pass:		# generate unit test if no explicit tests specified in abox arguments
                self.tests.append({'responses': [answer],
                                   'expected': ['correct'],
                                   })
        
        elif abtype == 'formularesponse':
            self.require_args(['expect', 'samples'])
            self.copy_attrib(abargs, 'inline', abxml)

            intype = self.stripquotes(abargs.get('intype', 'cs'))
            abxml.set('type', intype)

            self.copy_attrib(abargs, 'samples', abxml)
            
            if abargs.get('feqin'):
                tl = etree.Element('formulaequationinput')
            else:
                tl = etree.Element('textline')
            self.copy_attrib(abargs, 'trailing_text', tl)
            self.copy_attrib(abargs, 'size', tl)
            self.copy_attrib(abargs, 'inline', tl)
            self.copy_attrib(abargs, 'math', tl)
            self.copy_attrib(abargs, 'preprocessorClassName', tl)
            self.copy_attrib(abargs, 'preprocessorSrc', tl)
            abxml.append(tl)
            answer = self.stripquotes(abargs['expect'])
            abxml.set('answer', answer)
            rp = etree.SubElement(tl, "responseparam")
            rp.attrib['type'] = "tolerance"
            rp.attrib['default'] = abargs.get('tolerance') or "0.00001"
            if not self.has_test_pass:		# generate unit test if no explicit tests specified in abox arguments
                self.tests.append({'responses': [answer],
                                   'expected': ['correct'],
                                   })

        elif abtype == 'symbolicresponse':
            self.require_args(['expect'])
            self.copy_attrib(abargs, 'expect', abxml)
            self.copy_attrib(abargs, 'debug', abxml)
            self.copy_attrib(abargs, 'options', abxml)
            tl = etree.Element('textline')
            self.copy_attrib(abargs, 'inline', tl)
            self.copy_attrib(abargs, 'size', tl)
            self.copy_attrib(abargs, 'preprocessorClassName', tl)
            self.copy_attrib(abargs, 'preprocessorSrc', tl)
            self.copy_attrib(abargs, 'trailing_text', tl)
            abxml.append(tl)
            self.copy_attrib(abargs, 'inline', abxml)
            if 'correct_answer' in abargs:
                answer = self.stripquotes(abargs['correct_answer'])
            else:
                answer = self.stripquotes(abargs['expect'])
            tl.set('correct_answer', answer)
            tl.set('math', '1')  # use dynamath
            if not self.has_test_pass:		# generate unit test if no explicit tests specified in abox arguments
                self.tests.append({'responses': [answer],
                                   'expected': ['correct'],
                                   })
            
        elif abtype == 'imageresponse':
            self.require_args(['src', 'width', 'height', 'rectangle'])
            rect = abargs.get('rectangle')
            if re.match('\(\d+\,\d+\)\-\(\d+,\d+\)', rect) is None:  # check for rectangle syntax
                msg = "[abox.py] ERROR: imageresponse rectancle %s has wrong syntax\n" % rect
                msg += "Answer box string is \"%s\"\n" % self.aboxstr
                msg += "abox located: %s\n" % self.context
                raise Exception(msg)
                # sys.exit(-1)
            ii = etree.Element('imageinput')
            self.copy_attrib(abargs, 'src', ii)
            self.copy_attrib(abargs, 'width', ii)
            self.copy_attrib(abargs, 'height', ii)
            self.copy_attrib(abargs, 'rectangle', ii)
            abxml.append(ii)

        elif abtype=="config":
            '''
            Special case, used to set default configuration parameters.  Usage:

            \edXabox{type='config' for="custom" wrapclass="my_wrapper"}
            '''
            abxml = etree.Element("span")	# make this empty
            cfor = abargs.get('for')
            if cfor and cfor in type2response:
                params = abargs.copy()
                params.pop('type')
                params.pop('for')
                self.config[type2response[cfor]] = params
                print "[abox.py] Setting default parameters for %s to %s" % (cfor, params)
 
        # has hint function?
        if 'hintfn' in abargs:
            hintfn = self.stripquotes(abargs['hintfn'])
            hintgroup = etree.SubElement(abxml, 'hintgroup')
            hintgroup.set('hintfn', hintfn)

        # has hint?
        hint_extras = ''
        if 'hints' in abargs:
            hints = self.stripquotes(abargs['hints'])
            hintfn = "do_hints_for_%s" % hints
            hintgroup = etree.SubElement(abxml, 'hintgroup')
            hintgroup.set('hintfn', hintfn)
            hint_extras = "<edx_general_hint_system />\n"
            hint_extras += '<script type="text/python">\n%s = HintSystem(hints=%s).check_hint\n</script>\n' % (hintfn, hints)
        self.hint_extras = hint_extras

        xml_str = etree.tostring(abxml, pretty_print=True)
        xml_str = re.sub('(?ms)<html>(.*)</html>', '\\1', xml_str)
        # print s

        if script_code:
            code_str = '<script type="text/python" system_path="python_lib">\n'
            code_str += "<![CDATA[\n"
            code_str += script_code
            code_str += "\n]]>\n</script>\n"
            xml_str = "<span>%s\n%s</span>" % (xml_str, code_str)
            print "script code!"
            print xml_str

        the_xml = etree.XML(xml_str)

        return the_xml

    def get_options(self, abargs, arg='options'):
        optstr = abargs[arg]			# should be double quoted strings, comma delimited
        # EVH 01-22-2015: Inserting quotes around single option for proper
        # parsing of choices containing commas
        if not optstr.startswith('"') and not optstr.startswith("'"):
            optraw = repr(optstr)
            optstr = optraw[0] + optstr + optraw[0]
        options = split_args_with_quoted_strings(optstr, lambda(x): x == ',')		# turn into list of strings
        options = map(self.stripquotes, options)
        options = [x.strip() for x in options]		# strip strings
        if "" in options: options.remove("")
        optionstr = ','.join(["'%s'" % x for x in options])  # string of single quoted strings
        optionstr = "(%s)" % optionstr				# enclose in parens
        return optionstr, options
    
    def require_args(self, argnames):
        for argname in argnames:
            if argname not in self.abargs:
                msg = "============================================================\n"
                msg += "Error - abox requires %s argument\n" % argname
                msg += "Answer box string is \"%s\"\n" % self.aboxstr
                msg += "abox located: %s\n" % self.context
                # raise Exception, "Bad abox"
                raise Exception(msg)
                # sys.exit(-1)
            
    def process_test_arg(self, key, val):
        '''
        Record a unit test case.  These are specified by arguments like test_pass=...
        test_fail=..., test_spec=...
        '''
        test_args = map(self.stripquotes, split_args_with_quoted_strings(val, lambda(x): x == ','))
        test_args = map(self.unescape, test_args)
        if key=="test_spec":
            nargs = len(test_args)
            if not (nargs&1==0):
                raise Exception("[abox] test_spec must be given with an even number of subarguments, specifying equal number of responses and expected grader outputs")
            responses = test_args[:nargs/2]
            expected = test_args[nargs/2:]
            if 'correct' in expected:
                self.has_test_pass = True
        elif key=="test_pass":
            responses = test_args
            expected = "correct"
            self.has_test_pass = True
        elif key=="test_fail":
            responses = test_args
            expected = "incorrect"
        else:
            raise Exception("[abox] unknown test argument key %s" % key)
        test = {'responses': responses, 
                'expected': expected,
                'box_indexes': zip([0]*len(responses), range(len(responses))),
        }
        self.tests.append(test)

    def unescape(self, str):
        '''
        Unescape string which has been escaped for XML attribute encoding.
        Specifically, 
            &amp; -> &      &gt; -> >      &lt; -> <    
        '''
        return str.replace('&amp;', '&').replace('&gt;', '>').replace('&lt;', '<')

    def abox_args(self, s):
        '''
        Parse arguments of abox.  Splits by space delimitation.

        Test-spec argument keys are handled specially: test_*=...
        Arguments with those keys are stored in self.tests ; they may be used
        by the caller to construct answer box unit tests and course unit tests.
        '''
        s = s.replace(u'\u2019', "'")
        try:
            s = str(s)
        except Exception, err:
            print "Error %s in obtaining string form of abox argument %s" % (err, s)
            return {}
        try:
            # abargstxt = shlex.split(s)
            abargstxt = split_args_with_quoted_strings(s)
        except Exception, err:
            print "Error %s in parsing abox argument %s" % (err, s)
            return {}

        if '' in abargstxt:
            abargstxt.remove('')

        abargs = {}
        try:
            for key, val in [x.split('=', 1) for x in abargstxt]:
                if key.startswith("test_"):
                    self.process_test_arg(key, val)
                else:
                    abargs[key] = val
        except Exception, err:
            print "Error %s" % err
            print "Failed in parsing args = %s" % s
            print "abargstxt = %s" % abargstxt
            raise

        for arg in abargs:
            abargs[arg] = self.stripquotes(abargs[arg], checkinternal=True)

        return abargs

    def stripquotes(self, x, checkinternal=False):
        if x.startswith('"') and x.endswith('"'):
            if checkinternal and '"' in x[1:-1]:
                return x
            return x[1:-1]
        if x.startswith("'") and x.endswith("'"):
            return x[1:-1]
        return x

    def copy_attrib(self, abargs, aname, xml):
        if aname in abargs:
            xml.set(aname, self.stripquotes(abargs[aname]))

        
def split_args_with_quoted_strings(command_line, checkfn=None):
    """from pexpect.py
    This splits a command line into a list of arguments. It splits arguments
    on spaces, but handles embedded quotes, doublequotes, and escaped
    characters. It's impossible to do this with a regular expression, so I
    wrote a little state machine to parse the command line. """

    arg_list = []
    arg = ''

    if checkfn is None:
        def checkfn(c):
            return c.isspace()

    # Constants to name the states we can be in.
    state_basic = 0
    state_esc = 1
    state_singlequote = 2
    state_doublequote = 3
    # The state when consuming whitespace between commands.
    state_whitespace = 4
    state = state_basic

    for c in command_line:
        if state == state_basic or state == state_whitespace:
            if c == '\\':
                # Escape the next character
                state = state_esc
            elif c == r"'":
                # Handle single quote
                arg = arg + c
                state = state_singlequote
            elif c == r'"':
                # Handle double quote
                arg = arg + c
                state = state_doublequote
            elif checkfn(c):                  # OLD: c.isspace():
                # Add arg to arg_list if we aren't in the middle of whitespace.
                if state == state_whitespace:
                    # Do nothing.
                    None
                else:
                    arg_list.append(arg)
                    arg = ''
                    state = state_whitespace
            else:
                arg = arg + c
                state = state_basic
        elif state == state_esc:
            arg = arg + c
            state = state_basic
        elif state == state_singlequote:
            if c == r"'":
                arg = arg + c
                state = state_basic
            else:
                arg = arg + c
        elif state == state_doublequote:
            if c == r'"':
                arg = arg + c
                state = state_basic
            else:
                arg = arg + c

    if arg != '':
        arg_list.append(arg)
    return arg_list

#-----------------------------------------------------------------------------

def test_abox1():
    ab = AnswerBox('type="option" expect="float" options=" ","noneType","int","float"')
    print ab.xmlstr
    assert('''<optioninput options="('noneType','int','float')" correct="float"/>''' in ab.xmlstr)

def test_abox2():
    config = {}
    ab = AnswerBox('type="config" for="custom" wrapclass=mywrap.wrap(debug=True) import=mywrap', config=config)
    print ab.xmlstr
    print "config=%s" % config
    assert('''<span/>''' in ab.xmlstr)
    assert('customresponse' in config)

    ab = AnswerBox('type="custom" expect=10 cfn=mytest', config=config)
    print ab.xmlstr
    assert('''def cfn_wrap_''' in ab.xmlstr)

    # unset defaults
    ab = AnswerBox('type="config" for="custom"', config=config)
    print ab.xmlstr
    print "config=%s" % config
    assert('''<span/>''' in ab.xmlstr)
    assert('customresponse' in config)

    ab = AnswerBox('type="custom" expect=10 cfn=mytest', config=config)
    print ab.xmlstr
    assert('''def cfn_wrap_''' not in ab.xmlstr)

def test_abox_unit_test1():
    ab = AnswerBox('type="custom" expect=10 cfn=mytest test_pass=10')
    assert(ab.tests[0]['responses']==['10'])

def test_abox_unit_test2():
    ab = AnswerBox('type="custom" expect=10 cfn=mytest test_pass=10 test_fail=3')
    assert(len(ab.tests)==2)
    assert(ab.tests[0]['responses']==['10'])

def test_abox_unit_test3():
    the_err = None
    try:
        ab = AnswerBox('type="custom" expect=10 cfn=mytest test_pass=10 test_fail=3 test_bad=5')
    except Exception as err:
        the_err = err
    assert "unknown test argument key" in str(the_err)

def test_abox_unit_test4():
    ab = AnswerBox('type="custom" expect=10 cfn=mytest test_pass=10 test_fail=3 test_pass=11')
    print ab.tests
    assert(len(ab.tests)==3)
    assert(ab.tests[2]['responses']==['11'])
    assert(ab.tests[2]['expected']=='correct')

def test_abox_unit_test5():
    ab = AnswerBox('type="custom" expect=10 cfn=mytest test_spec=12,incorrect')
    assert(ab.tests[0]['responses']==['12'])
    assert(ab.tests[0]['expected']==['incorrect'])

def test_abox_unit_test6():
    ab = AnswerBox('type="custom" expect=10 cfn=mytest test_spec=12,10,incorrect,correct')
    assert(ab.tests[0]['responses']==['12',"10"])
    assert(ab.tests[0]['expected']==['incorrect',"correct"])

def test_abox_mc_ut1():
    ab = AnswerBox('type="multichoice" options="green","blue","red" expect="blue"')
    print ab.xmlstr
    assert('choicegroup' in ab.xmlstr)
    assert(ab.tests[0]['responses']==['choice_2'])
    assert(ab.tests[0]['expected']==['correct'])

def test_abox_mc_ut2():
    ab = AnswerBox('type="oldmultichoice" options="green","blue","red" expect="blue","red"')
    print ab.xmlstr
    assert('checkboxgroup' in ab.xmlstr)
    assert(ab.tests[0]['responses']==['choice_2', 'choice_3'])
    assert(ab.tests[0]['expected']==['correct', 'correct'])

def test_abox_option_ut1():
    ab = AnswerBox('type="option" options="green","blue","red" expect="blue"')
    print ab.xmlstr
    assert('optionresponse' in ab.xmlstr)
    assert(ab.tests[0]['responses']==['blue'])
    assert(ab.tests[0]['expected']==['correct'])
    
def test_abox_option_ut2():
    the_err = None
    try:
        ab = AnswerBox('type="option" options="green","blue","red" expect="orange"')
    except Exception as err:
        the_err = err
    assert("orange is not one of the options" in str(the_err))
    
def test_abox_custom_ut1():
    ab = AnswerBox('type="custom" expect="20" answers="11","9" prompts="Integer 1:","Integer 2:" inline="1" cfn="test_add"')
    assert('customresponse' in ab.xmlstr)
    assert(len(ab.tests)==1)
    assert(ab.tests[0]['responses']==['11', '9'])
    assert(ab.tests[0]['expected']==['correct', 'correct'])

def test_abox_custom_ut2():
    ab = AnswerBox('type="custom" expect="20" answers="11","9" prompts="Integer 1:","Integer 2:" inline="1" '
                   'cfn="test_add" test_fail="11","8" test_pass="10","10" '
                   'test_spec="7","13","correct","correct" ')
    assert('customresponse' in ab.xmlstr)
    assert(len(ab.tests)==3)
    assert(ab.tests[0]['responses']==['11', '8'])
    assert(ab.tests[0]['expected']=='incorrect')
    assert(ab.tests[1]['responses']==['10', '10'])
    assert(ab.tests[1]['expected']=='correct')
    assert(ab.tests[2]['responses']==['7', '13'])
    assert(ab.tests[2]['expected']==['correct']*2)
    assert(ab.tests[0]['box_indexes'] == [(0,0), (0,1)])

    



import contextlib
import os
import shutil
import tempfile


@contextlib.contextmanager
def make_temp_directory():
    temp_dir = tempfile.mkdtemp('l2etmp')
    current_dir = os.getcwd()
    yield temp_dir
    os.chdir(current_dir)
    shutil.rmtree(temp_dir)



'''
Test cross-reference and Table of Contents generating modules from
`latex2edx/main.py`
'''
import os
import re
import unittest
from lxml import etree
from path import path  # needs path.py

import latex2edx as l2e
from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class TestToC(unittest.TestCase):
    '''
    This class inherits the `unittest.TestCase` class and contains the methods
    `test_toc1` and `test_toc2` that test the proper functionality of the
    latex2edx `label`, `ref`, `toclabel`, and `tocref` commands.
    `test_toc3` tests the output tocindex.html file for the case when
    verticals are referenced.
    '''

    def test_toc1(self):
        '''
        Test the output of `latex2edx example11_toc_test.tex` for proper label
        and reference tags, as well as figure numbering.
        '''
        testdir = path(l2e.__file__).parent / 'testtex'
        tfn = testdir / 'example11_toc_test.tex'
        print "file %s" % tfn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, tfn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            l2eout = latex2edx(nfn, output_dir=tmdir)
            l2eout.convert()

            xhfn = nfn[:-4] + '.xhtml'
            xml = etree.fromstring(open(xhfn).read())
            toclabels = xml.findall('.//toclabel')
            self.assertEqual(toclabels[0].text, 'chap:intro')
            self.assertEqual(toclabels[1].text, 'mo:explore')
            tocref = xml.find('.//tocref')
            self.assertEqual(tocref.text, 'mo:explore')
            labels = xml.findall('.//label')
            self.assertEqual(labels[0].text, 'fig:single')
            self.assertEqual(labels[1].text, 'fig:multi')
            captions = xml.findall('.//div[@class="caption"]/b')
            self.assertEqual(captions[0].text, 'Figure 1')
            self.assertEqual(captions[1].text, 'Figure 2')
            chaps = xml.findall('.//chapter')
            self.assertIsNone(chaps[0].get('refnum'))
            self.assertEqual(chaps[1].get('refnum'), '1')
            equation = xml.find('.//td[@class="equation"]')
            self.assertEqual(re.findall(r'\\label\{(.*?)\}', equation.text,
                                        re.S)[0], 'eq:pythagorean')
            tocfn = '%s/tabs/tocindex.html' % tmdir
            toc = etree.fromstring(open(tocfn).read().replace('<br>', '<br/>'))
            self.assertEqual(toc.findtext('body/h1'), 'Table of Contents')
            # Check the measurable outcome table headers
            tocmos = toc.findall('.//table/tbody/tr/th/a')
            self.assertEqual(tocmos[0][0].text, '0')
            self.assertEqual(tocmos[1][0].text, 'MO1')
            self.assertEqual(tocmos[2][0].text, 'MO2')
            self.assertIn('Explore the edX platform',
                          tocmos[1].getnext().text)
            self.assertIn('Answer an edX question',
                          tocmos[2].getnext().text)
            # check measurable outcomes subheadings
            tocsubs = toc.find('.//table/tbody/tr/td')
            self.assertEqual(tocsubs[0].text, 'Learn')
            self.assertEqual(tocsubs[2].text, 'Assess')
            # check reference to measurable outcomes
            tocrefs = toc.findall('.//table/tbody/tr/td/ul/li/a')
            self.assertIn('Example problem', tocrefs[0].text)
            self.assertIn('Example text', tocrefs[1].text)
            self.assertIn('Example problem', tocrefs[2].text)
            self.assertIn('Example problem', tocrefs[3].text)
            self.assertEqual(toc[1][8].tag, 'br')
            self.assertIn('Module 1', toc[1][9][0].text)

    def test_toc2(self):
        '''
        Test the output of `latex2edx --popups example11_toc_test.tex` for
        proper equation numbering and button links to the static tocindex.html
        file.
        '''
        testdir = path(l2e.__file__).parent / 'testtex'
        tfn = testdir / 'example11_toc_test.tex'
        print "file %s" % tfn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, tfn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            l2eout = latex2edx(nfn, output_dir=tmdir, popup_flag=True)
            l2eout.convert()

            cfn = path(tmdir) / 'html/text-L1.xml'
            data = open(cfn).read()
            xml = etree.fromstring(data)
            # Check the first equation
            eqn = xml.find('.//table[@class="equation"]')
            self.assertEqual(eqn[0][0].text,
                             '[mathjax] \\frac{d}{dx} e^ x = e^ x[/mathjax]')
            # Check the reference link text
            href = xml.findall('.//a[@href]')
            self.assertEqual(href[0].text, '0')  # Non-numbered chapter
            self.assertEqual(href[1].text, '1.2')  # Numbered equation 2
            self.assertEqual(href[2].text, '1.3')  # Numbered equation 3
            # Check for popup format
            self.assertEqual(href[1].get('href'), 'javascript: void(0)')
            self.assertEqual(href[2].get('href'), 'javascript: void(0)')
            # Check for taglist in problem
            cfn = path(tmdir) / 'problem/p0.xml'
            data = open(cfn).read()
            xml = etree.fromstring(data)
            taglist = xml.find('.//p[@id="taglist"]')
            self.assertEqual(taglist.get('tags'),
                             'mo:explore,mo:problem,chap:intro')

    def test_toc3(self):
        '''
        Test the output of `latex2edx example17_toc_vert.tex` for proper
        generation of a tocindex.html file based on the contents of a vertical.
        '''
        testdir = path(l2e.__file__).parent / 'testtex'
        tfn = testdir / 'example17_toc_vert.tex'
        print "file %s" % tfn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, tfn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            l2eout = latex2edx(nfn, output_dir=tmdir)
            l2eout.convert()

            tocfn = '%s/tabs/tocindex.html' % tmdir
            toc = etree.fromstring(open(tocfn).read().replace('<br>', '<br/>'))
            self.assertEqual(toc.findtext('body/h1'), 'Table of Contents')
            # Check the measurable outcome table headers
            tocmos = toc.findall('.//table/tbody/tr/th/a')
            self.assertEqual(tocmos[0][0].text, 'MO1.1')
            self.assertEqual(tocmos[1][0].text, 'MO1.2')
            self.assertIn('Follow a Lesson', tocmos[0].getnext().text)
            self.assertIn('Answer a problem set', tocmos[1].getnext().text)
            # check measurable outcomes subheadings
            tocsubs = toc.find('.//table/tbody/tr/td')
            self.assertEqual(tocsubs[0].text, 'Learn')
            self.assertEqual(tocsubs[2].text, 'Assess')
            # check reference to measurable outcomes
            tocref1 = toc.find('.//table/tbody/tr[@id="indmo1p1"]/td/ul/li/a')
            self.assertIn('Example text', tocref1.text)
            tocref2 = toc.find('.//table/tbody/tr[@id="indmo1p2"]/td/ul/li/a')
            self.assertIn('Problem Set 1', tocref2.text)


if __name__ == '__main__':
    unittest.main()
